<?php

namespace IAWP_SCOPED\IAWP;

use IAWP_SCOPED\IAWP\Models\Visitor;
use IAWP_SCOPED\IAWP\Utils\Request;
class WooCommerce_Order
{
    private $order_id;
    private $total;
    private $total_refunded;
    private $total_refunds;
    private $status;
    /**
     * @param int $order_id WooCommerce order ID
     */
    public function __construct(int $order_id)
    {
        $order = wc_get_order($order_id);
        $this->order_id = $order_id;
        $this->total = \floatval($order->get_total());
        $this->total_refunded = \floatval($order->get_total_refunded());
        $this->total_refunds = \count($order->get_refunds());
        $this->status = $order->get_status();
    }
    /**
     * Insert or update a row in wp_independent_analytics_wc_orders
     *
     * @return void
     */
    public function upsert() : void
    {
        global $wpdb;
        $wc_orders_table = Query::get_table_name(Query::WC_ORDERS);
        $existing_wc_order = $wpdb->get_row($wpdb->prepare("SELECT * FROM {$wc_orders_table} WHERE order_id = %d", $this->order_id));
        if (!\is_null($existing_wc_order)) {
            $wpdb->update($wc_orders_table, ['total' => $this->total, 'total_refunded' => $this->total_refunded, 'total_refunds' => $this->total_refunds, 'status' => $this->status], ['order_id' => $this->order_id]);
            return;
        }
        $most_recent_view_id = $this->most_recent_view_id();
        if (\is_null($most_recent_view_id)) {
            return;
        }
        $wpdb->insert($wc_orders_table, ['order_id' => $this->order_id, 'view_id' => $most_recent_view_id, 'total' => $this->total, 'total_refunded' => $this->total_refunded, 'total_refunds' => $this->total_refunds, 'status' => $this->status, 'created_at' => (new \DateTime())->format('Y-m-d H:i:s')]);
    }
    /**
     * Get the ID of the most recent view so the order can be associated with that view
     *
     * @return int|null ID of the most recent view
     */
    private function most_recent_view_id() : ?int
    {
        $visitor = new Visitor(Request::ip(), Request::user_agent());
        return $visitor->most_recent_view_id();
    }
    public static function initialize_order_tracker()
    {
        \add_action('woocommerce_checkout_order_created', function ($order) {
            try {
                $woocommerce_order = new self($order->id);
                $woocommerce_order->upsert();
            } catch (\Throwable $e) {
                \error_log('Independent Analytics was unable to track the analytics for a WooCommerce order. Please report this error to Independent Analytics. The error message is below.');
                \error_log($e->getMessage());
            }
        });
        \add_action('woocommerce_analytics_update_order_stats', function ($order_id) {
            try {
                $woocommerce_order = new self($order_id);
                $woocommerce_order->upsert();
            } catch (\Throwable $e) {
                \error_log('Independent Analytics was unable to track the analytics for a WooCommerce order. Please report this error to Independent Analytics. The error message is below.');
                \error_log($e->getMessage());
            }
        });
    }
}
